class StatisticCalculator:
    def __init__(
        self,
        norm_res,
        norm_ved,
        norm_volume,
        norm_time,
        norm_seq,
        not_res,
        not_ved,
        not_volume,
        not_time,
        not_seq,
    ):
        self.norm_res = norm_res
        self.norm_ved = norm_ved
        self.norm_volume = norm_volume
        self.norm_time = norm_time
        self.norm_seq = norm_seq
        self.not_res = not_res
        self.not_ved = not_ved
        self.not_volume = not_volume
        self.not_time = not_time
        self.not_seq = not_seq

    def get_statistic_for_properties_and_all_stat(self):
        result_dict = dict()
        result_dict["Процент нормальных значений объёмов ресурсов"] = round(
            self.norm_res
        )
        if self.not_res != 100:
            result_dict["Процент нетипичных значений объёмов ресурсов"] = 100 - round(
                self.norm_res
            )
        else:
            result_dict["Процент нетипичных значений объёмов ресурсов"] = 0

        result_dict["Процент нормальных ресурсов по ведомостям"] = round(self.norm_ved)
        if self.not_ved != 100:
            result_dict["Процент нетипичных ресурсов по ведомостям"] = 100 - round(
                self.norm_ved
            )
        else:
            result_dict["Процент нетипичных ресурсов по ведомостям"] = 100
        result_dict["Процент нормальных значений времени работ"] = round(self.norm_time)
        if self.not_time != 100:
            result_dict["Процент нетипичных значений времени работ"] = 100 - round(
                self.norm_time
            )
        else:
            result_dict["Процент нетипичных значений времени работ"] = 0

        result_dict["Процент нормальных значений объёмов работ"] = round(
            self.norm_volume
        )

        if self.not_volume != 100:
            result_dict["Процент нетипичных значений объёмов работ"] = 100 - round(
                self.norm_volume
            )
        else:
            result_dict["Процент нетипичных значений объёмов работ"] = 0
        result_dict["Процент нормальных значений связей работ"] = round(self.norm_seq)
        if self.not_seq != 100:
            result_dict["Процент нетипичных значений связей работ"] = 100 - round(
                self.norm_seq
            )
        else:
            result_dict["Процент нетипичных значений связей работ"] = 0

        result_dict["Процент нормальных занчений по всем работам"] = round(
            (round(self.norm_time) + round(self.norm_volume) + round(self.norm_seq)) / 3
        )
        result_dict["Процент критических занчений по всем работам"] = 100 - round(
            (round(self.norm_time) + round(self.norm_volume) + round(self.norm_seq)) / 3
        )

        result_dict["Процент нормальных значений по всем ресурсам"] = round(
            (round(self.norm_res) + round(self.norm_ved)) / 2
        )
        result_dict["Процент критических значений по всем ресурсам"] = 100 - round(
            (round(self.norm_res) + round(self.norm_ved)) / 2
        )

        return result_dict

    def get_plan_statistic(self):
        norm_value = round(
            (
                self.norm_res
                + self.norm_ved
                + self.norm_volume
                + self.norm_time
                + self.norm_seq
            )
            / 5
        )
        crit_value = 100 - norm_value
        not_val = round(
            (
                self.not_res
                + self.not_ved
                + self.not_volume
                + self.not_time
                + self.not_seq
            )
            / 5
        )
        tested_val = 100 - not_val
        dict_result = {
            "Процент нормальных значений плана": norm_value,
            "Процент нетипичных значений плана": crit_value,
            "Процент непокрытых валидацией значений плана": not_val,
            "Процент покрытых валидацией значений плана": tested_val,
        }

        return dict_result



import numpy as np
import pandas as pd

from .BaseValidator import BaseValidator


class ResourcesValidator(BaseValidator):
    def __init__(self, res: list):
        super().__init__()
        self.res = res

    def validate(
        self, model_dataset: pd.DataFrame, df_wind_val: pd.DataFrame, act: list
    ):
        model_dataset = model_dataset.drop_duplicates()
        act = [c + "_act_fact" for c in act]
        df_perc_agg = pd.DataFrame()
        df_style = pd.DataFrame()
        df_volume = pd.DataFrame()
        fig_dict = dict()
        for i in model_dataset.index:
            c_pair = [ci for ci in act if model_dataset.loc[i, ci] != 0]
            c_act = c_pair
            volume = [model_dataset.loc[i, ci] for ci in c_act]
            delta = [self.percent_delta * volume_i for volume_i in volume]
            not_c = [ci for ci in act if ci not in c_act]
            zero_ind = df_wind_val[not_c][(df_wind_val[not_c] == 0).all(axis=1)].index
            sample_non = df_wind_val.loc[zero_ind, :]

            non_zero = sample_non[c_act][(sample_non[c_act] != 0).all(axis=1)].index
            sample_non = pd.DataFrame(sample_non.loc[non_zero, :])
            sample = pd.DataFrame()
            for j, ci in enumerate(c_act):
                sample = sample_non.loc[
                    (sample_non[ci] >= volume[j] - delta[j])
                    & (sample_non[ci] <= volume[j] + delta[j])
                ]
            sample = pd.DataFrame(sample)

            if sample.shape[0] > 4:
                for r in self.res:
                    value = model_dataset.loc[i, r]
                    q1, q99 = np.quantile(
                        sample[r].values, [self.lower_quantile, self.upper_quantile]
                    )
                    if value < q1 or value > q99:
                        df_style.loc[i, r] = "red"
                        df_volume.loc[i, r] = value
                        for ci in c_act:
                            key, blue_points, black_points, star = self._process_key(
                                c_act, r, ci, sample, sample_non, model_dataset, i
                            )

                            color = "red"
                            fig_dict[key] = {
                                "Blue points": blue_points,
                                "Black points": black_points,
                                "Star": star,
                                "Color": color,
                            }
                    else:
                        df_style.loc[i, r] = "green"
                        df_volume.loc[i, r] = value
                        for ci in c_act:
                            key, blue_points, black_points, star = self._process_key(
                                c_act, r, ci, sample, sample_non, model_dataset, i
                            )

                            color = "green"
                            fig_dict[key] = {
                                "Blue points": blue_points,
                                "Black points": black_points,
                                "Star": star,
                                "Color": color,
                            }
                df_style.loc[i, "Наименование"] = str(c_act)
                df_volume.loc[i, "Наименование"] = str(c_act)
            elif sample.shape[0] <= 4:
                df_style.loc[i, "Наименование"] = str(c_act)
                df_volume.loc[i, "Наименование"] = str(c_act)
                for r in self.res:
                    value = model_dataset.loc[i, r]
                    df_style.loc[i, r] = "grey"
                    df_volume.loc[i, r] = value
        new_df_color = df_style[(df_style != "grey").all(1)]
        not_perc = (
            (
                (df_style.shape[0] * df_style.shape[1])
                - (new_df_color.shape[0] * new_df_color.shape[1])
            )
            / (df_style.shape[0] * df_style.shape[1])
        ) * 100
        j = 0

        for c in act:
            new_sample = new_df_color.loc[
                new_df_color["Наименование"].str.count(c) != 0
            ]
            if new_sample.shape[0] != 0:
                for r in self.res:
                    df_perc_agg.loc[j, "Наименование ресурса"] = r
                    df_perc_agg.loc[j, "Наименование работы"] = c
                    value_dict = new_sample[r].value_counts().to_dict()
                    if "green" in list(value_dict.keys()):
                        df_perc_agg.loc[j, "Соотношение"] = round(
                            ((value_dict["green"]) / new_sample.shape[0]) * 100
                        )
                    else:
                        df_perc_agg.loc[j, "Соотношение"] = 0
                    j += 1
            else:
                for r in self.res:
                    df_perc_agg.loc[j, "Наименование ресурса"] = r
                    df_perc_agg.loc[j, "Наименование работы"] = c
                    df_perc_agg.loc[j, "Соотношение"] = 0
                    j += 1

        norm_perc = df_perc_agg["Соотношение"].mean()
        df_final_volume = pd.DataFrame()
        df_final_style = pd.DataFrame()
        for i, p in enumerate(list(df_volume["Наименование"].unique())):
            sample1 = df_volume.loc[df_volume["Наименование"] == p]
            sample2 = df_style.loc[df_style["Наименование"] == p]
            date = str(sample1.index[0]) + " " + str(sample1.index[-1])
            df_final_volume.loc[i, "Наименование"] = p
            df_final_volume.loc[i, "Даты"] = date
            df_final_volume.loc[i, self.res] = sample1.loc[sample1.index[0], self.res]
            df_final_style.loc[i, "Наименование"] = p
            df_final_style.loc[i, "Даты"] = date
            df_final_style.loc[i, self.res] = sample2.loc[sample2.index[0], self.res]

        return (
            df_perc_agg,
            df_final_volume,
            df_final_style,
            fig_dict,
            not_perc,
            norm_perc,
        )

    @staticmethod
    def _process_key(c_act, r, ci, sample, sample_non, model_dataset, i):
        key = str(c_act) + " " + r + " " + ci
        blue_points = {
            "x": list(sample_non[ci].values),
            "y": list(sample_non[r].values),
        }
        black_points = {
            "x": list(sample[ci].values),
            "y": list(sample[r].values),
        }
        star = {
            "x": model_dataset.loc[i, ci],
            "y": model_dataset.loc[i, r],
        }
        return key, blue_points, black_points, star



import numpy as np
import pandas as pd


class Aggregator:
    def __init__(self, brave_crit_value: float = 0.45):
        self.CRIT_VALUE = brave_crit_value

    @staticmethod
    def get_all_seq_statistic(start_end_data, model_data):
        # for i in start_end_data.index:
        #     act_name = start_end_data.loc[i, "smr_name"]
        #     if start_end_data.loc[i, "smr_name"] in act_dict_processed_swapped.keys():
        #         if (
        #             len(act_dict_processed_swapped[start_end_data.loc[i, "smr_name"]])
        #             > 1
        #         ):
        #             act_name = random.choice(
        #                 act_dict_processed_swapped[start_end_data.loc[i, "smr_name"]]
        #             )
        #         else:
        #             act_name = act_dict_processed_swapped[
        #                 start_end_data.loc[i, "smr_name"]
        #             ][0]
        #     start_end_data.loc[i, "smr_name"] = act_name

        works_dict = {
            act["activity_id"]: act["activity_name"] for act in model_data["activities"]
        }
        pairs_labels = {
            (act["activity_name"], works_dict[el[0]]): el[1]
            for act in model_data["activities"]
            if act["descendant_activities"]
            for el in act["descendant_activities"]
        }

        pairs = list(pairs_labels.keys())

        if not pairs:
            return pd.DataFrame(), pd.DataFrame(), 0, 0

        freq_dict = {}

        def check_dates(row):
            s1 = row["first_day_x"]
            f1 = row["last_day_x"]
            s2 = row["first_day_y"]
            f2 = row["last_day_y"]

            fs = int(f1 < s2)
            ss = int(s1 == s2)
            ff = int(f1 == f2)
            mix = int(s2 < f1)
            total = fs + ss + ff + mix

            return pd.Series(
                [fs, ss, ff, mix, total], index=["FS", "SS", "FF", "FFS", "count"]
            )

        for w1, w2 in pairs:
            ind1 = start_end_data.loc[(start_end_data["granular_smr_name"] == w1)]
            ind2 = start_end_data.loc[(start_end_data["granular_smr_name"] == w2)]
            merged_data = pd.merge(
                ind1, ind2, how="inner", on="upper_works", suffixes=("_x", "_y")
            )
            counts = pd.Series(
                [0, 0, 0, 0, 0], index=["FS", "SS", "FF", "FFS", "count"]
            )
            if merged_data.shape[0] != 0:
                counts = merged_data.apply(check_dates, axis=1)

            total_fs = counts["FS"].sum()
            total_ss = counts["SS"].sum()
            total_ff = counts["FF"].sum()
            total_ffs = counts["FFS"].sum()
            total_count = counts["count"].sum()

            if total_count > 0:
                freq_dict[w1, w2] = {
                    "count": total_count,
                    "FS": total_fs,
                    "SS": total_ss,
                    "FF": total_ff,
                    "FFS": total_ffs,
                }

        bar_records = []
        color_records = []
        links = ["FFS", "FS", "SS", "FF"]

        for i, ((w1, w2), label) in enumerate(pairs_labels.items()):
            pair_data = freq_dict.get((w1, w2), {})
            total_count = pair_data.get("count", 0)
            if total_count == 0:
                continue

            mix_val = pair_data.get("FFS", 0) / total_count
            fs_val = pair_data.get("FS", 0) / total_count
            ss_val = pair_data.get("SS", 0) / total_count
            ff_val = pair_data.get("FF", 0) / total_count
            links_perc = [mix_val, fs_val, ss_val, ff_val]
            max_label = links[np.argmax(links_perc)]

            bar_record = {
                "Наименование работы 1": w1,
                "Наименование работы 2": w2,
                "Связь в плане": label,
                "FFS": mix_val * 100,
                "FS": fs_val * 100,
                "SS": ss_val * 100,
                "FF": ff_val * 100,
            }
            bar_records.append(bar_record)

            color_record = {
                "Наименование работы 1": w1,
                "Наименование работы 2": w2,
                "color": "green" if max_label == label else "red",
            }
            color_records.append(color_record)

        df_bar = pd.DataFrame.from_records(bar_records)
        df_color = pd.DataFrame.from_records(color_records)

        perc_dict = df_color["color"].value_counts().to_dict()
        norm_perc = (perc_dict.get("green", 0) / df_color.shape[0]) * 100
        not_perc = 0

        return df_bar, df_color, norm_perc, not_perc

    def get_res_ved_stat(self, brave, ksg_for_val_data):
        df_stat = pd.DataFrame(columns=["Работа", "Ресурс", "Метка ресурса"])
        # new_index = []
        # new_columns = []
        # for r in brave_res:
        #     if r.split("_res_fact")[0] in res_dict_swapped.keys():
        #         if len(res_dict_swapped[r.split("_res_fact")[0]]) > 1:
        #             new_index.append(
        #                 random.choice(res_dict_swapped[r.split("_res_fact")[0]])
        #                 + "_res_fact"
        #             )
        #         else:
        #             new_index.append(
        #                 res_dict_swapped[r.split("_res_fact")[0]][0] + "_res_fact"
        #             )
        #     else:
        #         new_index.append(r)
        # for r in brave_act:
        #     if r.split("_act_fact")[0] in act_dict_processed_swapped.keys():
        #         if len(act_dict_processed_swapped[r.split("_act_fact")[0]]) > 1:
        #             new_columns.append(
        #                 random.choice(
        #                     act_dict_processed_swapped[r.split("_act_fact")[0]]
        #                 )
        #                 + "_act_fact"
        #             )
        #         else:
        #             new_columns.append(
        #                 act_dict_processed_swapped[r.split("_act_fact")[0]][0]
        #                 + "_act_fact"
        #             )
        #     else:
        #         new_columns.append(r)
        # brave.index = new_index
        # brave.columns = new_columns
        # brave = brave.groupby(brave.index).sum()
        # brave = brave.groupby(axis=1, level=0).sum()

        def evaluate_resource(row):
            df_row = pd.DataFrame(columns=["Работа", "Ресурс", "Метка ресурса"])
            res = row["labor_resources"]
            act_name = row["activity_name"]
            for i, r in enumerate(res):
                df_row.loc[i, "Работа"] = act_name
                df_row.loc[i, "Ресурс"] = r["labor_name"]
                df_row.loc[i, "Метка ресурса"] = "grey"
                if (
                    f"{row['activity_name']}_act_fact" in brave.columns
                    and f"{r['labor_name']}_res_fact" in brave.index
                ):
                    df_row.loc[i, "Метка ресурса"] = (
                        "green"
                        if brave.loc[
                            r["labor_name"] + "_res_fact",
                            row["activity_name"] + "_act_fact",
                        ]
                        >= self.CRIT_VALUE
                        else "red"
                    )

            return df_row

        for work in ksg_for_val_data["activities"]:
            df_result = evaluate_resource(work)
            df_stat = pd.concat([df_stat, df_result], ignore_index=True)
        # df_stat = df_stat.append(
        #   pd.DataFrame(map(evaluate_resource, ksg_for_val_data["activities"])), ignore_index=True
        # )

        not_grey = df_stat[df_stat["Метка ресурса"] != "grey"]
        not_perc = (len(df_stat) - len(not_grey)) / len(df_stat) * 100
        norm_perc = (
            not_grey["Метка ресурса"].value_counts().to_dict().get("green", 0)
            / len(not_grey)
        ) * 100

        return df_stat, not_perc, norm_perc



import numpy as np
import pandas as pd

from .BaseValidator import BaseValidator


class TimeValidator(BaseValidator):
    def validate(
        self, df_wind_model: pd.DataFrame, df_wind_val: pd.DataFrame, act: list
    ):
        df_wind_model = df_wind_model.drop_duplicates()
        df_stat = pd.DataFrame()
        dict_fig = {}
        final_df = pd.DataFrame()
        j = 0
        for c in act:
            dict_fig[c] = []
            for i in df_wind_model.index:
                if df_wind_model.loc[i, c] != 0:
                    c_act = [c]
                    volume = [df_wind_model.loc[i, ci] for ci in c_act]
                    delta = [self.percent_delta * volume_i for volume_i in volume]
                    sample = df_wind_val.copy()
                    for k, ci in enumerate(c_act):
                        sample = sample.loc[
                            (sample[ci] >= volume[k] - delta[k])
                            & (sample[ci] <= volume[k] + delta[k])
                        ]
                    if sample.shape[0] > 3:
                        df_stat, dict_for_sample = self.handle_sample(
                            i,
                            c,
                            df_wind_model,
                            sample,
                            df_wind_val,
                            df_stat,
                            j,
                        )
                        dict_fig[c].append(
                            {
                                "volume": volume,
                                "fig_data": dict_for_sample,
                                "color": dict_for_sample["Color"],
                            }
                        )
                    else:
                        df_stat.loc[j, "Работа"] = c
                        df_stat.loc[j, "Метка времени"] = "grey"
                    j += 1

        not_grey = df_stat.loc[df_stat["Метка времени"] != "grey"]
        not_perc = ((df_stat.shape[0] - not_grey.shape[0]) / df_stat.shape[0]) * 100
        norm_df = df_stat.loc[df_stat["Метка времени"] == "green"]
        norm_perc = 0
        if not_perc != 100:
            norm_perc = (
                (not_grey.shape[0] - norm_df.shape[0]) / not_grey.shape[0]
            ) * 100

        final_df = self.finalize_dataframe(act, final_df, not_grey)
        return final_df, dict_fig, norm_perc, not_perc

    def handle_sample(self, i, c, df_wind_model, sample, df_wind_val, df_stat, j):
        value = df_wind_model.loc[i, c.split("_act_fact")[0] + "_real_time_act"]
        q1, q99 = np.quantile(
            sample[c.split("_act_fact")[0] + "_real_time_act"].values,
            [self.lower_quantile, self.upper_quantile],
        )
        q1 = int(q1)
        q99 = int(q99)
        if value < q1 or value > q99:
            color = "red"
        else:
            color = "green"
        df_stat.loc[j, "Работа"] = c
        df_stat.loc[j, "Метка времени"] = color
        sample_dict = self.create_figure_dict(
            c, color, sample, df_wind_val, df_wind_model, i, q1, q99
        )
        return df_stat, sample_dict

    @staticmethod
    def create_figure_dict(
        c,
        color,
        sample,
        df_wind_val: pd.DataFrame,
        df_wind_model: pd.DataFrame,
        i,
        q1,
        q99,
    ):
        blue_points = {
            "x": list(df_wind_val[c].values),
            "y": list(df_wind_val[c.split("_act_fact")[0] + "_real_time_act"].values),
        }
        black_points = {
            "x": list(sample[c].values),
            "y": list(sample[c.split("_act_fact")[0] + "_real_time_act"].values),
        }
        star = {
            "x": df_wind_model.loc[i, c],
            "y": df_wind_model.loc[i, c.split("_act_fact")[0] + "_real_time_act"],
        }
        return {
            "Blue points": blue_points,
            "Black points": black_points,
            "Star": star,
            "Color": color,
            "Q1": q1,
            "Q99": q99,
        }

    @staticmethod
    def finalize_dataframe(act: list, final_df, not_grey):
        for i, c in enumerate(act):
            final_df.loc[i, "Наименование"] = c
            sample = not_grey.loc[not_grey["Работа"] == c]
            count_dict = sample["Метка времени"].value_counts().to_dict()
            if "green" not in list(count_dict.keys()):
                final_df.loc[i, "Время на ед. объёма"] = 0
            else:
                final_df.loc[i, "Время на ед. объёма"] = (
                    count_dict["green"] / sample.shape[0]
                ) * 100
        return final_df



from spprval.dataset.BaseDataset import BaseDataset
import pandas as pd
import datetime


class ValidationDataset(BaseDataset):
    def __init__(
        self,
        data_source,
        pools,
        act_names,
        res_names,
    ):
        self.data_source = data_source
        self.pools = pools
        self.act_names = act_names
        self.res_names = res_names

    def collect(self):
        validation_dataset_list = self.data_source.get_data(self.pools, self.res_names)
        validation_dataset = pd.DataFrame()
        for df in validation_dataset_list:
            validation_dataset = pd.concat([validation_dataset, df])

        validation_dataset.fillna(0, inplace=True)
        df_pools = {}
        for pool in self.pools:
            df_pools[str(pool)] = []
            for _, df in validation_dataset.groupby("object_name"):
                work_set = set(df["granulary_name"].values)
                if set(pool).issubset(work_set):
                    df = df[["is_work", "granulary_name"] + list(df.columns[8:])]
                    df = df.groupby("granulary_name").sum()
                    df = df.reset_index()
                    df_work = df.loc[df["is_work"] == True]
                    sum_work = df.loc[df["granulary_name"].isin(pool)][
                        ["granulary_name"] + list(df.columns[2:])
                    ]  # это я беру имя работ + даты, всё остальное мне не нужно
                    sum_work = sum_work.loc[
                        :, (sum_work != 0).all(axis=0)
                    ]  # тут удаляю работы, у которых все объёмы за все даты нулевые
                    if len(sum_work.columns) > 1:
                        d1 = dict(df_work[df_work.columns[2:]].sum())
                        d2 = dict(sum_work[sum_work.columns[1:]].sum())
                        shared_items = {k: d2[k] for k in d2 if d1[k] == d2[k]}
                        if len(shared_items) != 0:
                            res = df.loc[df["is_work"] == False][
                                ["granulary_name"] + list(shared_items.keys())
                            ]
                            sum_work = sum_work[
                                ["granulary_name"] + list(shared_items.keys())
                            ]
                            sum_work = pd.concat([sum_work, res])
                            df_pools[str(pool)].append(sum_work)
        final_df = pd.DataFrame()
        for p in df_pools:
            for df in df_pools[p]:
                df = df.transpose()
                df.columns = df.iloc[0]
                df.drop(index=df.index[0], axis=0, inplace=True)
                df = df.rename_axis(None, axis=1)
                df.reset_index(drop=True, inplace=True)
                final_df.reset_index(drop=True, inplace=True)
                final_df = pd.concat([final_df, df], ignore_index=True)

        base = datetime.datetime.today()
        date_list = [
            (base + datetime.timedelta(days=x)).strftime("%d.%m.%Y")
            for x in range(final_df.shape[0])
        ]
        final_df.index = date_list
        final_df.fillna(0, inplace=True)
        for c in self.act_names:
            if c not in final_df.columns:
                final_df[c] = 0
        for r in self.res_names:
            if r not in final_df.columns:
                final_df[r] = 0

        new_cols = []
        for c in final_df.columns:
            if c in self.act_names:
                new_cols.append(c + "_act_fact")
            else:
                new_cols.append(c + "_res_fact")
        final_df.columns = new_cols
        return final_df



import matplotlib.pyplot as plt
import numpy as np
import pandas as pd

from .BaseValidator import BaseValidator


class WorksValidator(BaseValidator):
    def validate(
        self, model_dataset: pd.DataFrame, df_wind_val: pd.DataFrame, act: list
    ):
        model_dataset = model_dataset.drop_duplicates()
        df_stat = pd.DataFrame()
        dist_dict = dict()
        j = 0
        for c in act:
            for i in model_dataset.index:
                value = model_dataset.loc[i, c]
                if value != 0:
                    sample = df_wind_val.loc[df_wind_val[c] != 0]
                    if sample.shape[0] != 0:
                        q1, q99 = np.quantile(
                            sample[c].values, [self.lower_quantile, self.upper_quantile]
                        )
                        q1 = int(q1)
                        q99 = int(q99)
                        if value < q1 or value > q99:
                            df_stat.loc[j, "Работа"] = c
                            df_stat.loc[j, "Метка работы"] = "red"
                            key = c
                            line = value
                            color = "red"
                            counts, bins, _ = plt.hist(sample[c].values)
                            dist_dict[key] = {
                                "Line": line,
                                "color": color,
                                "Hight": counts,
                                "Bins": bins,
                                "Q1": q1,
                                "Q99": q99,
                            }
                        else:
                            df_stat.loc[j, "Работа"] = c
                            df_stat.loc[j, "Метка работы"] = "green"
                            key = c
                            line = value
                            color = "green"
                            counts, bins, _ = plt.hist(sample[c].values)
                            dist_dict[key] = {
                                "Line": line,
                                "color": color,
                                "Hight": counts,
                                "Bins": bins,
                                "Q1": q1,
                                "Q99": q99,
                            }
                        j += 1
                    else:
                        df_stat.loc[j, "Работа"] = c
                        df_stat.loc[j, "Метка работы"] = "grey"
        not_grey = df_stat.loc[df_stat["Метка работы"] != "grey"]
        not_perc = ((df_stat.shape[0] - not_grey.shape[0]) / df_stat.shape[0]) * 100
        norm_df = df_stat.loc[df_stat["Метка работы"] == "green"]
        norm_perc = ((not_grey.shape[0] - norm_df.shape[0]) / not_grey.shape[0]) * 100
        df_final_stat = pd.DataFrame()
        for i, c in enumerate(act):
            df_final_stat.loc[i, "Наименование"] = c
            sample = not_grey.loc[not_grey["Работа"] == c]
            count_dict = sample["Метка работы"].value_counts().to_dict()
            if "green" not in list(count_dict.keys()):
                df_final_stat.loc[i, "Среднедневная выработка"] = 0
            else:
                df_final_stat.loc[i, "Среднедневная выработка"] = (
                    count_dict["green"] / sample.shape[0]
                ) * 100
        return df_final_stat, dist_dict, norm_perc, not_perc



import datetime
import json
from copy import copy
from os import listdir
from os.path import isfile, join

import pandas as pd
import pkg_resources

from spprval.database.DBWrapper import DBWrapper
from spprval.dataset.KSGDataset import KSGDataset
from spprval.dataset.PlanDataset import PlanDataset
from spprval.dataset.ValidationDataset import ValidationDataset
from .aggregator import Aggregator
from .expert_metrics_estimator import ExpertMetricsEstimator
from .stats_calculator import StatisticCalculator
from .validators.ResourcesValidator import ResourcesValidator
from .validators.TimeValidator import TimeValidator
from .validators.WorksValidator import WorksValidator

brave_precalculated = "tables/precalculated_brave_coefficients.csv"
links_history = "tables/links_history_messoyaha_with_granular_smr_v4.csv"
brave_path = pkg_resources.resource_filename(__name__, brave_precalculated)
links_history_path = pkg_resources.resource_filename(__name__, links_history)


class PlanValidator:
    def __init__(self, plan, url):
        self.ksg_data = KSGDataset(ksg_data=plan).collect()
        self.plan_dataset = PlanDataset(ksg_data=self.ksg_data)
        self.data_source = DBWrapper(url=url)
        self.aggregator = Aggregator()
        self.works_validator = WorksValidator()
        self.time_validator = TimeValidator()
        self.resources_validator = None
        self.statistic_calculator = None

    def validate(self):
        plan_df = self.plan_dataset.collect()
        act = self.plan_dataset.get_act_names()
        res = self.plan_dataset.get_res_names()

        brave = pd.read_csv(brave_path, index_col=0)

        pools = self.plan_dataset.get_pools()

        validation_df = ValidationDataset(self.data_source, pools, act, res).collect()

        print("Валидируем ресурсы")
        self.resources_validator = ResourcesValidator(
            res=[r + "_res_fact" for r in res]
        )
        (
            df_perc_agg_res,
            df_final_volume_res,
            df_final_style_res,
            fig_dict_res,
            not_perc_res,
            norm_perc_res,
        ) = self.resources_validator.validate(plan_df, validation_df, act)
        print("Валидация ресурсов по ведомости")
        df_vedom, not_perc_vedom, norm_perc_vedom = self.aggregator.get_res_ved_stat(
            brave, self.ksg_data
        )
        print("Нарезка данных на окна")
        _, df_wind_val = self.window_zero(validation_df)
        _, df_wind_model = self.window_zero(plan_df)
        for i in df_wind_val.index:
            for c in act:
                df_wind_val.loc[i, c.split("_")[0] + "_real_time_act"] = (
                    df_wind_val.loc[i, "Window length"]
                    - df_wind_val.loc[i, c.split("_")[0] + "_act_fact_zero_lil"]
                    - df_wind_val.loc[i, c.split("_")[0] + "_act_fact_zero_big"]
                )
        for i in df_wind_model.index:
            for c in act:
                df_wind_model.loc[i, c.split("_")[0] + "_real_time_act"] = (
                    df_wind_model.loc[i, "Window length"]
                    - df_wind_model.loc[i, c.split("_")[0] + "_act_fact_zero_lil"]
                    - df_wind_model.loc[i, c.split("_")[0] + "_act_fact_zero_big"]
                )

        print("Валидация среднедневных объёмов работ")
        (
            df_volume_stat,
            dist_dict,
            norm_volume_perc,
            not_volume_perc,
        ) = self.works_validator.validate(
            plan_df, validation_df, act=[c + "_act_fact" for c in act]
        )
        print("Валидация времени")
        (
            df_time_stat,
            time_plot_dict,
            norm_time_perc,
            not_time_perc,
        ) = self.time_validator.validate(
            df_wind_model, df_wind_val, act=[c + "_act_fact" for c in act]
        )
        print("Валидация последовательностей")
        seq_history = pd.read_csv(links_history_path)
        (
            df_seq_stat,
            df_color_seq,
            norm_perc_seq,
            not_perc_seq,
        ) = self.aggregator.get_all_seq_statistic(seq_history, self.ksg_data)
        print("Получение общих статистик по работам и ресурсам")
        self.statistic_calculator = StatisticCalculator(
            norm_perc_res,
            norm_perc_vedom,
            norm_volume_perc,
            norm_time_perc,
            norm_perc_seq,
            not_perc_res,
            not_perc_vedom,
            not_volume_perc,
            not_time_perc,
            not_perc_seq,
        )
        work_res_common_perc = (
            self.statistic_calculator.get_statistic_for_properties_and_all_stat()
        )
        plan_statistic = self.statistic_calculator.get_plan_statistic()

        return (
            df_perc_agg_res,
            df_final_volume_res,
            df_final_style_res,
            fig_dict_res,
            df_vedom,
            df_volume_stat,
            dist_dict,
            df_time_stat,
            time_plot_dict,
            df_seq_stat,
            df_color_seq,
            work_res_common_perc,
            plan_statistic,
        )

    @staticmethod
    def window_zero(df: pd.DataFrame, min_window: int = 2, max_window: int = 10):
        df = df.drop_duplicates()
        base = datetime.datetime.today()
        date_list = [
            (base + datetime.timedelta(days=x)).strftime("%d.%m.%Y")
            for x in range(df.shape[0])
        ]
        df.index = date_list
        """Function for making time windows of msg files

        Args:
            df (pd.DataFrame): input msg file
            min_window (int, optional): Min length of window. Defaults to 5.
            max_window (int, optional): Max length of window. Defaults to 31.

        Returns:
            DataFrame: msg files with windows
        """
        act_col = [var for var in df.columns if "_act_fact" in var]

        res_col = [var for var in df.columns if "_res_fact" in var]
        new_zero_lil = [act + "_zero_lil" for act in act_col]
        new_zero_big = [act + "_zero_big" for act in act_col]
        df_new = pd.DataFrame(
            columns=list(df.columns)
            + ["Start day", "Window length"]
            + new_zero_lil
            + new_zero_big
        )
        df_zero = pd.DataFrame(columns=act_col)
        for window_len in [7]:
            for start in range(0, len(df) - window_len + 1):
                new_row = (
                    df[act_col]
                    .iloc[start : start + window_len]
                    .sum(axis=0, skipna=True)
                )
                new_row = {
                    act: 1 if value == 0.0 else 0 for act, value in new_row.items()
                }
                start_day = datetime.datetime.strptime(df.index[start], "%d.%m.%Y")
                end_day = datetime.datetime.strptime(
                    df.index[start + window_len - 1], "%d.%m.%Y"
                )
                df_zero.loc[df.index[start]] = new_row

        window_list = list(range(min_window, max_window + 1))
        for window_len in window_list:
            for start in range(0, len(df) - window_len + 1):
                start_day = datetime.datetime.strptime(df.index[start], "%d.%m.%Y")
                end_day = datetime.datetime.strptime(
                    df.index[start + window_len - 1], "%d.%m.%Y"
                )
                if (end_day - start_day).days == window_len - 1:
                    new_row_act = (
                        df[act_col]
                        .iloc[start : start + window_len]
                        .sum(axis=0, skipna=True)
                    )
                    new_row_res = (
                        df[res_col]
                        .iloc[start : start + window_len]
                        .mean(axis=0, skipna=True)
                    )
                    new_row = copy(new_row_act)
                    new_row = pd.concat([new_row, new_row_res])

                    lil = {act + "_zero_lil": 0 for act in act_col}
                    big = {act + "_zero_big": 0 for act in act_col}
                    danger_date = [
                        df.index[ind] for ind in range(start, start + window_len)
                    ]

                    for act in act_col:
                        for i in range(start, start + window_len):
                            if df.iloc[i][act] == 0.0:
                                lil[act + "_zero_lil"] = lil[act + "_zero_lil"] + 1
                                if (
                                    (i < window_len - 6)
                                    and danger_date[i - start] in df_zero.index
                                    and df_zero.loc[danger_date[i - start]][act] == 1
                                ):
                                    if (i > start) and (
                                        df_zero.loc[danger_date[i - start - 1]][act]
                                        == 1
                                    ):
                                        big[act + "_zero_big"] = (
                                            big[act + "_zero_big"] + 1
                                        )
                                    else:
                                        big[act + "_zero_big"] = (
                                            big[act + "_zero_big"] + 7
                                        )
                    lil = {
                        act
                        + "_zero_lil": lil[act + "_zero_lil"]
                        - big[act + "_zero_big"]
                        for act in act_col
                    }

                    new_row = {
                        **new_row,
                        **lil,
                        **big,
                        "Start day": start_day,
                        "Window length": window_len,
                    }
                    df_new = pd.concat(
                        [df_new, pd.DataFrame(pd.Series(new_row)).transpose()],
                        ignore_index=True,
                    )

        return df_zero, df_new

    @staticmethod
    def _file_opener(mypath, validation_files, act):
        onlyfiles = [f for f in listdir(mypath) if isfile(join(mypath, f))]
        for file in onlyfiles:
            msg_file = open(mypath + file, encoding="utf8")
            msg_data = json.load(msg_file)
            if len(msg_data["resource"]) != 0:
                for work in msg_data["work"]:
                    if work["work title"] in act:
                        validation_files.append(mypath + file)
                        break
        return validation_files

    @staticmethod
    def _save_json_file(label, json_file, new_plan_path):
        with open(new_plan_path + str(label) + ".json", "w") as outfile:
            json.dump(json_file, outfile)
        return new_plan_path + str(label) + ".json"


class ProjectValidation:
    def __init__(self, project_ksg, url):
        self.project_ksg = project_ksg
        self.url = url

    def validate_plan(self):
        result_dict = dict()
        ksg_for_val_data = self.project_ksg
        regime = True
        for el in ksg_for_val_data["activities"]:
            if (
                el["activity_name"] == "Окончание работ по марке"
                or el["activity_name"] == "Начало работ по марке"
            ):
                regime = False
                break
        if regime:
            (
                df_perc_agg_res,
                df_final_volume_res,
                df_final_style_res,
                fig_dict_res,
                df_vedom,
                df_volume_stat,
                dist_dict,
                df_time_stat,
                time_plot_dict,
                df_seq_stat,
                df_color_seq,
                work_res_common_perc,
                plan_statistic,
            ) = PlanValidator(plan=self.project_ksg, url=self.url).validate()
            result_dict["Object 0"] = {
                "Отношения ресурсов к работам": df_perc_agg_res,
                "Пулы работ объёмы": df_final_volume_res,
                "Пулы работ цвета": df_final_style_res,
                "Данные для графиков ресурсов": fig_dict_res,
                "Ресурсная ведомость": df_vedom,
                "Статистика по выработке": df_volume_stat,
                "Данные для графиков выработок": dist_dict,
                "Статистика по времени работ": df_time_stat,
                "Данные для графиков времени": time_plot_dict,
                "Таблица связей": df_seq_stat,
                "Таблица связей цвета": df_color_seq,
                "Статистика итоговая по работам и ресурсам": work_res_common_perc,
                "Итоговая статистика плана": plan_statistic,
            }
        else:
            json_files = []
            act_dict = dict()
            block_id = dict()
            ind_act = dict()
            label = 0
            for i, el in enumerate(ksg_for_val_data["activities"]):
                act_dict[el["activity_id"]] = el["activity_name"]
                if el["activity_name"] == "Окончание работ по марке":
                    label += 1
                elif el["activity_name"] == "Начало работ по марке":
                    label += 1
                else:
                    block_id[el["activity_id"]] = label
                    ind_act[el["activity_id"]] = i
            block_id["0000000"] = label + 1
            json_file = {"activities": []}
            label = block_id[list(block_id.keys())[0]]
            for k in block_id.keys():
                if block_id[k] == label:
                    ind_of_act = ind_act[k]
                    new_des_act = []
                    for el in ksg_for_val_data["activities"][ind_of_act][
                        "descendant_activities"
                    ]:
                        if el[0] in block_id.keys():
                            if (
                                act_dict[el[0]] != "Окончание работ по марке"
                                and act_dict[el[0]] != "Начало работ по марке"
                            ):
                                if block_id[k] == block_id[el[0]]:
                                    new_des_act.append(el)
                    ksg_for_val_data["activities"][ind_of_act][
                        "descendant_activities"
                    ] = new_des_act
                    json_file["activities"].append(
                        ksg_for_val_data["activities"][ind_of_act]
                    )
                    label = block_id[k]
                else:
                    json_files.append(json_file)
                    if k == "0000000":
                        break
                    else:
                        json_file = {"activities": []}
                        ind_of_act = ind_act[k]
                        new_des_act = []
                        for el in ksg_for_val_data["activities"][ind_of_act][
                            "descendant_activities"
                        ]:
                            if el[0] in block_id.keys():
                                if (
                                    act_dict[el[0]] != "Окончание работ по марке"
                                    and act_dict[el[0]] != "Начало работ по марке"
                                ):
                                    if block_id[k] == block_id[el[0]]:
                                        new_des_act.append(el)
                        ksg_for_val_data["activities"][ind_of_act][
                            "descendant_activities"
                        ] = new_des_act
                        json_file["activities"].append(
                            ksg_for_val_data["activities"][ind_of_act]
                        )
                        label = block_id[k]
            for id_number, p in enumerate(json_files):
                (
                    df_perc_agg_res,
                    df_final_volume_res,
                    df_final_style_res,
                    fig_dict_res,
                    df_vedom,
                    df_volume_stat,
                    dist_dict,
                    df_time_stat,
                    time_plot_dict,
                    df_seq_stat,
                    df_color_seq,
                    work_res_common_perc,
                    plan_statistic,
                ) = PlanValidator(plan=p, url=self.url).validate()
                result_dict["Object " + str(id_number)] = {
                    "Отношения ресурсов к работам": df_perc_agg_res,
                    "Пулы работ объёмы": df_final_volume_res,
                    "Пулы работ цвета": df_final_style_res,
                    "Данные для графиков ресурсов": fig_dict_res,
                    "Ресурсная ведомость": df_vedom,
                    "Статистика по выработке": df_volume_stat,
                    "Данные для графиков выработок": dist_dict,
                    "Статистика по времени работ": df_time_stat,
                    "Данные для графиков времени": time_plot_dict,
                    "Таблица связей": df_seq_stat,
                    "Таблица связей цвета": df_color_seq,
                    "Статистика итоговая по работам и ресурсам": work_res_common_perc,
                    "Итоговая статистика плана": plan_statistic,
                }
        return result_dict

    def calculate_expert_metrics(self):
        metrics_calculator = ExpertMetricsEstimator(self.project_ksg)
        metrics = metrics_calculator.calculate_metrics()
        return metrics

    @staticmethod
    def _filter_activities(k, block_id, act_dict, ksg_for_val_data, ind_act):
        ind_of_act = ind_act[k]
        new_des_act = []
        for el in ksg_for_val_data["activities"][ind_of_act]["descendant_activities"]:
            if el[0] in block_id.keys():
                if act_dict[el[0]] not in [
                    "Окончание работ по марке",
                    "Начало работ по марке",
                ]:
                    if block_id[k] == block_id[el[0]]:
                        new_des_act.append(el)
        ksg_for_val_data["activities"][ind_of_act][
            "descendant_activities"
        ] = new_des_act
        return ksg_for_val_data["activities"][ind_of_act]



def style(cell_value):
    default = ""

    if "Типичное значение" in cell_value:
        return "background-color: green;"
    if "Редкое значение" in cell_value:
        return "background-color: yellow;"
    if "Критическое значение" in cell_value:
        return "background-color: red;"
    if "Недостаточно исторических данных" in cell_value:
        return "background-color: grey;"
    return default


def style_vedom(cell_value):
    default = ""

    if "Connect by ved with high Brave" in cell_value:
        return "background-color: green;"
    if "Connect by ved with low Brave" in cell_value:
        return "background-color: red;"
    if "Not connect by ved with high Brave" in cell_value:
        return "background-color: violet;"
    if "Not connect by ved with low Brave" in cell_value:
        return "background-color: blue;"
    if "Resource not in data" in cell_value or "Work not in data" in cell_value:
        return "background-color: grey;"
    return default


def create_res_html(df_style, name, result_path):
    df_map_style = df_style.style.applymap(style)
    df_html = df_map_style.render()
    with open(result_path + name + "/" + "res_report.html", "w") as file:
        file.write(df_html)



from spprval.database.DBWrapper import DBWrapper
import pandas as pd


class ObjectNameProcessor:
    def __init__(self, db_url="http://10.32.15.31:8000"):
        self.db = DBWrapper(db_url)

    def _get_names(self, name_type: str) -> pd.DataFrame:
        if name_type == "act":
            return self.db.get_act_names()
        elif name_type == "res":
            return self.db.get_res_names()
        else:
            raise ValueError(f"Unsupported name_type: {name_type}")

    def _create_dict(self, names, name_type: str, name_field: str, swap: bool = False):
        df_all_names = self._get_names(name_type)
        name_dict = df_all_names.set_index("name").to_dict()[name_field]
        result_dict = {}
        for name in names:
            if name in name_dict:
                if swap:
                    if name_dict[name] in result_dict:
                        result_dict[name_dict[name]].append(name)
                    else:
                        result_dict[name_dict[name]] = [name]
                else:
                    if name in result_dict:
                        result_dict[name].append(name_dict[name])
                    else:
                        result_dict[name] = [name_dict[name]]
            else:
                result_dict[name] = [name]
        result_dict.update((k, k) for k, v in result_dict.items() if v == "-")
        return result_dict

    def create_processed_dict(self, names, name_type: str, swap: bool = False):
        return self._create_dict(names, name_type, "processed_name", swap)

    def create_granulary_dict(self, names, name_type: str, swap: bool = False):
        return self._create_dict(names, name_type, "granulary_name", swap)



from datetime import datetime


class ExpertMetricsEstimator:
    def __init__(self, project_data):
        self.project = project_data

    def _get_all_descendant_acts(self, index=1):
        return [
            desc[index]
            for act in self.project["activities"]
            for desc in act["descendant_activities"]
        ]

    def _calculate_percentage(self, condition, index=1):
        all_descendant_acts = self._get_all_descendant_acts(index)
        matching_count = sum(1 for desc in all_descendant_acts if condition(desc))
        return matching_count / len(all_descendant_acts) if all_descendant_acts else 0

    def follower_preds(self):
        descendant_percentage = self._calculate_percentage(
            lambda x: x not in self._get_all_descendant_acts(0), 0
        )
        return "Да" if descendant_percentage <= 0.05 else "Нет"

    def outrun(self):
        fss_percentage = self._calculate_percentage(lambda x: x == "FFS")
        return "Да" if fss_percentage <= 0.05 else "Нет"

    def finish_start(self):
        fs_ffs_percentage = self._calculate_percentage(lambda x: x in ["FS", "FFS"])
        return "Да" if fs_ffs_percentage >= 0.85 else "Нет"

    def others(self):
        other_percentage = self._calculate_percentage(lambda x: x not in ["FS", "FFS"])
        return "Да" if other_percentage <= 0.15 else "Нет"

    def crit_index(self):
        acts_start_date = datetime.strptime(
            self.project["activities"][0]["start_date"], "%Y-%m-%d"
        )
        acts_deadline_date = datetime.strptime(
            self.project["plan_deadline"], "%Y-%m-%d"
        )
        acts_end_date = datetime.strptime(
            self.project["activities"][-1]["start_date"], "%Y-%m-%d"
        )
        ldeadline = (acts_deadline_date - acts_start_date).days
        lcp = (acts_end_date - acts_start_date).days
        index = ldeadline / lcp
        return "Да" if index >= 1 else "Нет"

    def calculate_metrics(self):
        return {
            "follower_preds": self.follower_preds(),
            "outrun": self.outrun(),
            "finish_start": self.finish_start(),
            "others": self.others(),
            "crit_index": self.crit_index(),
        }



from spprval.dataset.BaseDataset import BaseDataset
from spprval.dataset.ObjectNameProcessor import ObjectNameProcessor


class KSGDataset(BaseDataset):
    def __init__(self, ksg_data):
        self.ksg_data = ksg_data

    def collect(self):
        for iter_act, w in enumerate(self.ksg_data["activities"]):
            if w["activity_name"] == "":
                self.ksg_data["activities"].pop(iter_act)
        dict_class = ObjectNameProcessor()
        act = []
        for w in self.ksg_data["activities"]:
            act.append(w["activity_name"])
        res = []
        for w in self.ksg_data["activities"]:
            for r in w["labor_resources"]:
                if r["labor_name"] not in res:
                    res.append(r["labor_name"])
        act_dict = dict_class.create_granulary_dict(act, "act")
        res_dict = dict_class.create_granulary_dict(res, "res")
        plan_dict = {}
        for act in self.ksg_data["activities"]:
            name = act["activity_name"]
            smr_name = act["activity_name"]
            if name in act_dict:
                smr_name = act_dict[name][0]
            if smr_name in plan_dict:
                plan_dict[smr_name].append(name)
            else:
                plan_dict[smr_name] = [name]
        delete_act = []
        for el in plan_dict:
            if len(plan_dict[el]) > 1:
                for c in plan_dict[el][1:]:
                    delete_act.append(c)
        new_act = []
        deleted_id = []
        for w in self.ksg_data["activities"]:
            if (float(w["volume"]) != 0.0) and (w["activity_name"] not in delete_act):
                new_act.append(w)
            else:
                deleted_id.append(w["activity_id"])
        self.ksg_data["activities"] = new_act
        for w in self.ksg_data["activities"]:
            new_des = []
            for el in w["descendant_activities"]:
                if el[0] not in deleted_id:
                    new_des.append(el)
            w["descendant_activities"] = new_des
        for w in self.ksg_data["activities"]:
            if w["activity_name"] in act_dict:
                w["activity_name"] = act_dict[w["activity_name"]][0]
            for r in w["labor_resources"]:
                if r["labor_name"] in res_dict:
                    r["labor_name"] = res_dict[r["labor_name"]][0]
        return self.ksg_data



import datetime
import json
import warnings
from datetime import timedelta

import pandas as pd

warnings.simplefilter(action="ignore", category=pd.errors.PerformanceWarning)


act_df = pd.read_csv("unique_tasks_dict.csv", sep=";")
act_dict = dict()
for i in act_df.index:
    act_dict[act_df.loc[i, "task_name"]] = act_df.loc[i, "unique_task_name"]


res_ved_dict = {
    "Агрегат сварочный_res_fact": "АПС (агрегат передвижной сварочный)_res_fact",
    "Бурильная установка_res_fact": "Бурильная машина_res_fact",
    "Водители, Машинисты, Механизаторы_res_fact": "Машинист, водители_res_fact",
    "Водители, механизаторы_res_fact": "Машинист, водители_res_fact",
    "Водитель (Газ)_res_fact": "Машинист, водители_res_fact",
    "Водитель (КАМАЗ автокран)_res_fact": "Машинист, водители_res_fact",
    "Водитель (КАМАЗ миксер)_res_fact": "Машинист, водители_res_fact",
    "Водитель (КАМАЗ площадка)_res_fact": "Машинист, водители_res_fact",
    "Водитель (савок)_res_fact": "Машинист, водители_res_fact",
    "Водитель_res_fact": "Машинист, водители_res_fact",
    "Грейдер_res_fact": "Автогрейдер_res_fact",
    "Кран стреловой_res_fact": "Автокран_res_fact",
    "Кран_res_fact": "Автокран_res_fact",
    "Кровельщик, изолировщик, арматурщик, бетонщик_res_fact": "Изолировщик_res_fact",
    "Кровельщик, изолировщик, арматурщик, отделочник, сантехник, отделочник_res_fact": "Изолировщик_res_fact",
    "Кровельщик, изолировщик, арматурщик, отделочник, сантехник_res_fact": "Изолировщик_res_fact",
    "Маш копра _res_fact": "Машинист, водители_res_fact",
    "Машинист ДЭС (дизельная электростанция)_res_fact": "Машинист, водители_res_fact",
    "Машинист ППУ (паровой передвижной установки)_res_fact": "Машинист, водители_res_fact",
    "Машинист Сваебойной установки_res_fact": "Машинист, водители_res_fact",
    "Машинист трубоукладчика_res_fact": "Машинист, водители_res_fact",
    "Машинист экскаватора_res_fact": "Машинист, водители_res_fact",
    "Механизатор_res_fact": "Машинист, водители_res_fact",
    "Монтажник ВЛ_res_fact": "Монтажник_res_fact",
    "Монтажник электромонтажа_res_fact": "Монтажник_res_fact",
    "Погрузчик телескопический Dieci Icarus_res_fact": "Погрузчик_res_fact",
    "Погрузчик телескопический_res_fact": "Погрузчик_res_fact",
    "Погрузчик фронтальный CAT_res_fact": "Погрузчик_res_fact",
    "Погрузчик-экскаватор Гидромик_res_fact": "Погрузчик_res_fact",
    "Помошник бурильщика_res_fact": "Бурильщик, помощник бурильщика_res_fact",
    "Сварочный агрегат_res_fact": "АПС (агрегат передвижной сварочный)_res_fact",
    "Сварочный дизель-генератор_res_fact": "АПС (агрегат передвижной сварочный)_res_fact",
    "Сварочный комплекс_res_fact": "АПС (агрегат передвижной сварочный)_res_fact",
    "Сварщик ЭМУ _res_fact": "Сварщик_res_fact",
    "Сварщик ЭМУ_res_fact": "Сварщик_res_fact",
    "Сварщики РД (труба)_res_fact": "Сварщик_res_fact",
    "Сварщики РД_res_fact": "Сварщик_res_fact",
    "Сварщики СК (общестрой)_res_fact": "Сварщик_res_fact",
    "Сварщики по технологии_res_fact": "Сварщик_res_fact",
    "Сварщики_res_fact": "Сварщик_res_fact",
    "Седельный тягач (площадка)_res_fact": "Тягач, тягач седельный, трал_res_fact",
    "Седельный тягач (трал)_res_fact": "Тягач, тягач седельный, трал_res_fact",
    "Седельный тягач_res_fact": "Тягач, тягач седельный, трал_res_fact",
    "Слесарь АМЦ_res_fact": "Слесарь_res_fact",
    "Трактор_res_fact": "Трактор гусеничный_res_fact",
    "Трал_res_fact": "Тягач, тягач седельный, трал_res_fact",
    "Тягач_res_fact": "Тягач, тягач седельный, трал_res_fact",
    'Экскаватор  "Komatsu" movax_res_fact': "Экскаватор_res_fact",
    "Экскаватор оборудованный бурильным оборудованием_res_fact": "Экскаватор_res_fact",
    "Электрик, электромонтажник_res_fact": "Электромонтажник_res_fact",
    "Электрик_res_fact": "Электромонтажник_res_fact",
}


res_dict_all = {
    " Трубоукладчик ТГ-122Я_res_fact": "Трубоукладчик_res_fact",
    "АБС (миксер)_res_fact": "Бетономешалка_res_fact",
    "АДД_res_fact": "АДД (дизельный сварочный агрегат)_res_fact",
    "Автобетоносмеситель Урал 583100_res_fact": "Бетономешалка_res_fact",
    "Автобетоносмеситель_res_fact": "Бетономешалка_res_fact",
    "Автобетоносмиситель_res_fact": "Бетономешалка_res_fact",
    "Автокран  25 тн_res_fact": "Автокран_res_fact",
    'Автокран "Grove GMK 4100L"_res_fact': "Автокран_res_fact",
    'Автокран "Liebher LTM 1100-4.2"_res_fact': "Автокран_res_fact",
    "Автокран 50тн_res_fact": "Автокран_res_fact",
    "Автокран _res_fact": "Автокран_res_fact",
    "Автокран, до 25_res_fact": "Автокран_res_fact",
    "Автокраны и гидроманипуляторы_res_fact": "Автокран_res_fact",
    "Автомобильный кран_res_fact": "Автокран_res_fact",
    "Агрегат сварочный_res_fact": "АПС (агрегат передвижной сварочный)_res_fact",
    "Бетоносмеситель_res_fact": "Бетономешалка_res_fact",
    "Бортовой Урал_res_fact": "Бортовой автомобиль_res_fact",
    "Бортовой автомобиль ГАЗ_res_fact": "Бортовой автомобиль_res_fact",
    'Бульдозер "CAT D8R"_res_fact': "Бульдозер_res_fact",
    'Бульдозер "Четра" Т-11_res_fact': "Бульдозер_res_fact",
    "Бульдозер _res_fact": "Бульдозер_res_fact",
    "Бульдозер типа Dressta (с рыхлителем)_res_fact": "Бульдозер_res_fact",
    "Бурилка_res_fact": "Бурильная машина_res_fact",
    "Бурильная машина БМ-811М_res_fact": "Бурильная машина_res_fact",
    "Бурильная установка_res_fact": "Бурильная машина_res_fact",
    'Бурильно-сваебойная "NIPPON"_res_fact': "Бурильная машина_res_fact",
    "Бурильно-сваебойная БМ811_res_fact": "Бурильная машина_res_fact",
    "Бурильно-сваебойный комплекс (на базе УРАЛ, КАМАЗ)_res_fact": "Бурильная машина_res_fact",
    "Бурильщик, помошник бурильщика_res_fact": "Бурильщик, помощник бурильщика_res_fact",
    "Бурильщик, помощник бурильщика_res_fact": "Бурильщик, помощник бурильщика_res_fact",
    "Бурильщик_res_fact": "Бурильщик, помощник бурильщика_res_fact",
    "Буровая машина БМ_res_fact": "Бурильная машина_res_fact",
    "Буровая машина_res_fact": "Бурильная машина_res_fact",
    "Буровая установка БМ 811_res_fact": "Бурильная машина_res_fact",
    "Буровая установка БМ_res_fact": "Бурильная машина_res_fact",
    "Буровая установка ТСГ_res_fact": "Бурильная машина_res_fact",
    "Буровая установка_res_fact": "Бурильная машина_res_fact",
    "Водители, Машинисты, Механизаторы_res_fact": "Машинист, водители_res_fact",
    "Водители, механизаторы_res_fact": "Машинист, водители_res_fact",
    "Водители_res_fact": "Машинист, водители_res_fact",
    "Водитель (Газ)_res_fact": "Машинист, водители_res_fact",
    "Водитель (КАМАЗ автокран)_res_fact": "Машинист, водители_res_fact",
    "Водитель (КАМАЗ миксер)_res_fact": "Машинист, водители_res_fact",
    "Водитель (КАМАЗ площадка)_res_fact": "Машинист, водители_res_fact",
    "Водитель (савок)_res_fact": "Машинист, водители_res_fact",
    "Водитель_res_fact": "Машинист, водители_res_fact",
    "Геодезист _res_fact": "Геодезист_res_fact",
    "Грейдер_res_fact": "Автогрейдер_res_fact",
    "ДЭС  АД-100с-Т400-1РМ1_res_fact": "ДЭС (дизельная электростанция)_res_fact",
    "ДЭС  АД-60с-Т400-1РМ1_res_fact": "ДЭС (дизельная электростанция)_res_fact",
    "ДЭС (дизельная электростанция)_res_fact": "ДЭС (дизельная электростанция)_res_fact",
    "ДЭС SDMO-D110_res_fact": "ДЭС (дизельная электростанция)_res_fact",
    "ДЭС SDMO-D350_res_fact": "ДЭС (дизельная электростанция)_res_fact",
    "ДЭС45_res_fact": "ДЭС (дизельная электростанция)_res_fact",
    "ДЭС_res_fact": "ДЭС (дизельная электростанция)_res_fact",
    "Деффектоскопист_res_fact": "Дефектоскопист_res_fact",
    "Дизель электростанция_res_fact": "ДЭС (дизельная электростанция)_res_fact",
    "Дизельная электростанция ДЭС-100_res_fact": "ДЭС (дизельная электростанция)_res_fact",
    "ИТР_res_fact": "ИТР (инженерно-технический персонал)_res_fact",
    "Изолеровщик_res_fact": "Изолировщик_res_fact",
    "Изолировщики_res_fact": "Изолировщик_res_fact",
    "Кран на автомобильном ходу г/п 25 тн на шасси Урал 4320 КС 45717_res_fact": "Автокран_res_fact",
    "Кран на гусеничном ходу РДК 250_res_fact": "Автокран_res_fact",
    "Кран стреловой_res_fact": "Автокран_res_fact",
    "Кран электрический СКГ-401_res_fact": "Автокран_res_fact",
    "Кран-манипулятор_res_fact": "Автокран_res_fact",
    "Кран_res_fact": "Автокран_res_fact",
    "Краны (автокран)_res_fact": "Автокран_res_fact",
    "Краны (автокран, RDK)_res_fact": "Автокран_res_fact",
    "Кровельщик, изолировщик, арматурщик, бетонщик_res_fact": "Изолировщик_res_fact",
    "Кровельщик, изолировщик, арматурщик, отделочник, сантехник, отделочник_res_fact": "Изолировщик_res_fact",
    "Кровельщик, изолировщик, арматурщик, отделочник, сантехник_res_fact": "Изолировщик_res_fact",
    "Маляр _res_fact": "Маляр_res_fact",
    "Маляры_res_fact": "Маляр_res_fact",
    "Маш СП-49_res_fact": "Машинист, водители_res_fact",
    "Маш копра _res_fact": "Машинист, водители_res_fact",
    "Маш. ДЭС_res_fact": "Машинист, водители_res_fact",
    "Маш. ППУ_res_fact": "Машинист, водители_res_fact",
    "Маш. трубоукладчика_res_fact": "Машинист, водители_res_fact",
    "Маш. экскаватора _res_fact": "Машинист, водители_res_fact",
    "Машинист_res_fact": "Машинист, водители_res_fact",
    "Машинисты, водители_res_fact": "Машинист, водители_res_fact",
    "Машинисты,водители_res_fact": "Машинист, водители_res_fact",
    "Машинисты_res_fact": "Машинист, водители_res_fact",
    "Механизатор_res_fact": "Машинист, водители_res_fact",
    "Механизаторы, водители, пом. машиниста бур. уст-ки_res_fact": "Машинист, водители_res_fact",
    "Механизаторы, водители, техник учет ГСМ_res_fact": "Машинист, водители_res_fact",
    "Механизаторы_res_fact": "Машинист, водители_res_fact",
    "Механик _res_fact": "Машинист, водители_res_fact",
    "Механик_res_fact": "Машинист, водители_res_fact",
    "Монтажник ВЛ_res_fact": "Электромонтажник_res_fact",
    "Монтажник ТТ_res_fact": "Монтажник ТТ (технологических трубопроводов)_res_fact",
    "Монтажник м/к_res_fact": "Монтажник МК (металлоконструкций)_res_fact",
    "Монтажник электромонтажа_res_fact": "Электромонтажник_res_fact",
    "Монтажники (СК)_res_fact": "Монтажник_res_fact",
    "Монтажники (труба)_res_fact": "Монтажник ТТ (технологических трубопроводов)_res_fact",
    "Монтажники стальных и ж/б конструкций_res_fact": "Монтажник МК (металлоконструкций)_res_fact",
    "Монтажники_res_fact": "Монтажник_res_fact",
    "Одноковшовый экскаватор емкостью не менее 1.2 куб.м_res_fact": "Экскаватор_res_fact",
    "ПСК (Передвижной Сварочный Комплекс) _res_fact": "АПС (агрегат передвижной сварочный)_res_fact",
    "ПСК (Передвижной Сварочный Комплекс) Камаз_res_fact": "АПС (агрегат передвижной сварочный)_res_fact",
    "ПСК (передвижной сварочный комплекс)_res_fact": "АПС (агрегат передвижной сварочный)_res_fact",
    "Передвижной сварочный агрегат АС-42_res_fact": "АПС (агрегат передвижной сварочный)_res_fact",
    "Передвижной сварочный комплекс_res_fact": "АПС (агрегат передвижной сварочный)_res_fact",
    "Погрузчик телескопический Dieci Icarus_res_fact": "Погрузчик_res_fact",
    "Погрузчик телескопический_res_fact": "Погрузчик_res_fact",
    "Погрузчик фронтальный CAT_res_fact": "Погрузчик_res_fact",
    "Погрузчик-экскаватор Гидромик_res_fact": "Погрузчик_res_fact",
    "Пом. Бур _res_fact": "Бурильщик, помошник бурильщика_res_fact",
    "Пом. машиниста бур. уст-ки_res_fact": "Бурильщик, помошник бурильщика_res_fact",
    "Помошник бурильщика_res_fact": "Бурильщик, помошник бурильщика_res_fact",
    "Разнорабочие_res_fact": "Разнорабочий_res_fact",
    "Разнорабочий, комендант, кладовщик_res_fact": "Разнорабочий_res_fact",
    "Сварочные посты_res_fact": "АПС (агрегат передвижной сварочный)_res_fact",
    "Сварочный агрегат Denyo_res_fact": "АПС (агрегат передвижной сварочный)_res_fact",
    "Сварочный агрегат на базе трактора_res_fact": "АПС (агрегат передвижной сварочный)_res_fact",
    "Сварочный агрегат(дуговая сварка, полуавтоматическая,автоматическая,плазморезы и пр.)_res_fact": "АПС (агрегат передвижной сварочный)_res_fact",
    "Сварочный ап. 2-хпост диз. На шасси_res_fact": "АПС (агрегат передвижной сварочный)_res_fact",
    "Сварочный аппарат  1 пост_res_fact": "АПС (агрегат передвижной сварочный)_res_fact",
    "Сварочный аппарат  10 пост ВДМ_res_fact": "АПС (агрегат передвижной сварочный)_res_fact",
    "Сварочный аппарат  1пост_res_fact": "АПС (агрегат передвижной сварочный)_res_fact",
    "Сварочный аппарат  2хпост_res_fact": "АПС (агрегат передвижной сварочный)_res_fact",
    "Сварочный аппарат  4-хпост ВДМ_res_fact": "АПС (агрегат передвижной сварочный)_res_fact",
    "Сварочный дизель-генератор_res_fact": "АПС (агрегат передвижной сварочный)_res_fact",
    "Сварочный комплекс_res_fact": "АПС (агрегат передвижной сварочный)_res_fact",
    "Сварщик ТТ_res_fact": "Сварщик ТТ (технологических трубопроводов)_res_fact",
    "Сварщик ЭМУ _res_fact": "Сварщик МК (металлоконструкций)_res_fact",
    "Сварщик ЭМУ_res_fact": "Сварщик МК (металлоконструкций)_res_fact",
    "Сварщик м/к_res_fact": "Сварщик МК (металлоконструкций)_res_fact",
    "Сварщик электромонтажа_res_fact": "Сварщик МК (металлоконструкций)_res_fact",
    "Сварщик_res_fact": "Сварщик МК (металлоконструкций)_res_fact",
    "Сварщики STT+М300 (труба)_res_fact": "Сварщик ТТ (технологических трубопроводов)_res_fact",
    "Сварщики РД (труба)_res_fact": "Сварщик ТТ (технологических трубопроводов)_res_fact",
    "Сварщики РД_res_fact": "Сварщик ТТ (технологических трубопроводов)_res_fact",
    "Сварщики СК (общестрой)_res_fact": "Сварщик ТТ (технологических трубопроводов)_res_fact",
    "Сварщики металлоконструкций_res_fact": "Сварщик МК (металлоконструкций)_res_fact",
    "Сварщики по технологии_res_fact": "Сварщик ТТ (технологических трубопроводов)_res_fact",
    "Сварщики_res_fact": "Сварщик ТТ (технологических трубопроводов)_res_fact",
    "Стропальщики_res_fact": "Стропальщик_res_fact",
    "Стропольщик_res_fact": "Стропальщик_res_fact",
    "Трубоукладчик _res_fact": "Трубоукладчик_res_fact",
    'Тягач  "Камаз"_res_fact': "Тягач, тягач седельный, трал_res_fact",
    "Тягач седельный_res_fact": "Тягач, тягач седельный, трал_res_fact",
    "Тягач сидельный_res_fact": "Тягач, тягач седельный, трал_res_fact",
    "Тягач, тягач седельный, трал_res_fact": "Тягач, тягач седельный, трал_res_fact",
    "Тягач_res_fact": "Тягач, тягач седельный, трал_res_fact",
    "Тягачи седельные_res_fact": "Тягач, тягач седельный, трал_res_fact",
    'Экскаватор  "Komatsu" movax_res_fact': "Экскаватор_res_fact",
    'Экскаватор  "Komatsu" вращатель_res_fact': "Экскаватор_res_fact",
    'Экскаватор  "Komatsu" ковш_res_fact': "Экскаватор_res_fact",
    "Экскаватор  HUNDAI R 250 LC-7 ковш_res_fact": "Экскаватор_res_fact",
    "Экскаватор (вращатель)_res_fact": "Экскаватор_res_fact",
    "Экскаватор (ковшевой)_res_fact": "Экскаватор_res_fact",
    "Экскаватор _res_fact": "Экскаватор_res_fact",
    "Экскаватор оборудованный бурильным оборудованием _res_fact": "Экскаватор_res_fact",
    "Экскаватор оборудованный бурильным оборудованием_res_fact": "Экскаватор_res_fact",
    "Экскватор _res_fact": "Экскаватор_res_fact",
    "Эл. монтажник_res_fact": "Электромонтажник_res_fact",
    "Электрик, электромонтажник_res_fact": "Электромонтажник_res_fact",
    "Электромонтажник по силовым сетям_res_fact": "Электромонтажник_res_fact",
    "Электромонтажник, электромонтер_res_fact": "Электромонтажник_res_fact",
    "Электромонтажники_res_fact": "Электромонтажник_res_fact",
    "Электромонтер_res_fact": "Электромонтажник_res_fact",
    "Электросварщик_res_fact": "Сварщик МК (металлоконструкций)_res_fact",
    "Электросварщики_res_fact": "Сварщик МК (металлоконструкций)_res_fact",
    "Электростанция (ДЭС) (50,100,200,400 кВт)_res_fact": "ДЭС (дизельная электростанция)_res_fact",
}


config = pd.read_csv("Config_of_params.csv")

BRAVE_CRIT = config.loc[0, "Crit value of Brave"]


def create_model_dataset(ksg_for_val_data):
    """Function to convert json file to msg file

    Args:
        ksg_for_val_data (dict): input json file
    Returns:
        DataFrame: msg file
    """
    model_dataset = pd.DataFrame()
    for w in ksg_for_val_data["activities"]:
        name = w["activity_name"]

        start = datetime.datetime.strptime(w["start_date"].split()[0], "%Y-%m-%d")
        end = datetime.datetime.strptime(w["end_date"].split()[0], "%Y-%m-%d")
        vol = float(w["volume"])
        res_data = dict()
        for r in w["labor_resources"]:
            if "_res_fact" in r["labor_name"]:
                res_data[r["labor_name"]] = r["volume"]
            else:
                res_data[r["labor_name"] + "_res_fact"] = r["volume"]
        days = (end - start).days + 1
        vol_per = vol / days
        delta = timedelta(days=1)
        while start <= end:
            model_dataset.loc[start, name] = vol_per
            for k in res_data.keys():
                model_dataset.loc[start, k] = res_data[k]
            start += delta
    model_dataset.fillna(0, inplace=True)
    model_dataset.index = model_dataset.index.strftime("%d.%m.%Y")
    model_columns = []
    for c in model_dataset.columns:
        if "_res_fact" not in c:
            model_columns.append(c + "_act_fact")
        else:
            model_columns.append(c)
    model_dataset.columns = model_columns

    return model_dataset


def get_work_pools(ksg_for_val_data):
    """Function for obtaining pools of parallel jobs in the KSG

    Args:
        ksg_for_val_data (dict): input json file

    Returns:
        list: list of pools
    """
    act = []
    for w in ksg_for_val_data["activities"]:
        act.append(w["activity_name"])
    model_dataset = pd.DataFrame()
    for w in ksg_for_val_data["activities"]:
        name = w["activity_name"]
        start = datetime.datetime.strptime(w["start_date"].split()[0], "%Y-%m-%d")
        end = datetime.datetime.strptime(w["end_date"].split()[0], "%Y-%m-%d")
        vol = float(w["volume"])
        days = (end - start).days + 1
        vol_per = vol / days
        delta = timedelta(days=1)
        while start <= end:
            model_dataset.loc[start, name] = vol_per
            start += delta
    model_dataset.fillna(0, inplace=True)
    work_pools = []
    for i in model_dataset.index:
        pool = []
        for c in model_dataset.columns:
            if model_dataset.loc[i, c] != 0:
                pool.append(c)
        work_pools.append(pool)
    work_pools_final = []
    for p in work_pools:
        if p not in work_pools_final and len(p) != 0:
            work_pools_final.append(p)
    return work_pools_final


def get_validation_dataset(val_files, pools, act, res):
    """Function for generating a validation dataset

    Args:
        val_files (list): list of json files names which contain acts from ksg
        pools (list): list of work pools
        act (list): list of works
        res (list): list of resources

    Returns:
        DataFrame: validation dataset
    """
    validation_dataset = pd.DataFrame()
    msg_df = dict()
    for file in val_files:
        f = open(file, encoding="utf8")
        msg_json = json.load(f)
        for w in msg_json["work"]:
            name = w["work title"]
            if name in act_dict.keys():
                name = act_dict[name]
            if name:
                for w_d in w["work_data"]["progress"]:
                    if name not in msg_df.keys():
                        msg_df[name] = dict()
                        msg_df[name][list(w_d.keys())[0]] = float(
                            w_d[list(w_d.keys())[0]]["fact"]
                        )
                    else:
                        if list(w_d.keys())[0] in msg_df[name].keys():
                            msg_df[name][list(w_d.keys())[0]] += float(
                                w_d[list(w_d.keys())[0]]["fact"]
                            )
                        else:
                            msg_df[name][list(w_d.keys())[0]] = float(
                                w_d[list(w_d.keys())[0]]["fact"]
                            )
        for r in msg_json["resource"]:
            name = r["resource_name"]
            if name:
                if name + "_res_fact" in res_dict_all.keys():
                    name = res_dict_all[name + "_res_fact"].split("_")[0]
                elif name + "_res_fact" in res_ved_dict.keys():
                    name = res_ved_dict[name + "_res_fact"].split("_")[0]
                for r_d in r["progress"]:
                    if name not in msg_df.keys():
                        msg_df[name] = dict()
                        msg_df[name][list(r_d.keys())[0]] = float(
                            r_d[list(r_d.keys())[0]]["fact"]
                        )
                    else:
                        if list(r_d.keys())[0] in msg_df[name].keys():
                            msg_df[name][list(r_d.keys())[0]] += float(
                                r_d[list(r_d.keys())[0]]["fact"]
                            )
                        else:
                            msg_df[name][list(r_d.keys())[0]] = float(
                                r_d[list(r_d.keys())[0]]["fact"]
                            )

                    # if name in msg_df.columns:
                    #     if list(r_d.keys())[0] in msg_df.index:
                    #         if pd.isna(msg_df.loc[list(r_d.keys())[0], name]) or pd.isnull(msg_df.loc[list(r_d.keys())[0], name]):
                    #             try:
                    #                 msg_df.loc[list(r_d.keys())[0], name] = float(r_d[list(r_d.keys())[0]]['fact'])
                    #             except:
                    #                 msg_df.loc[list(r_d.keys())[0], name] = 0
                    #         else:
                    #             try:
                    #                 msg_df.loc[list(r_d.keys())[0], name] += float(r_d[list(r_d.keys())[0]]['fact'])
                    #             except:
                    #                 msg_df.loc[list(r_d.keys())[0], name] = 0
                    #     else:
                    #         try:
                    #             msg_df.loc[list(r_d.keys())[0], name] = float(r_d[list(r_d.keys())[0]]['fact'])
                    #         except:
                    #             msg_df.loc[list(r_d.keys())[0], name] = 0
                    # else:
                    #     try:
                    #         msg_df.loc[list(r_d.keys())[0], name] = float(r_d[list(r_d.keys())[0]]['fact'])
                    #     except:
                    #         msg_df.loc[list(r_d.keys())[0], name] = 0

    # for c in act:
    #     if c in msg_df.columns:

    #         for i in msg_df.index:
    #             validation_dataset.loc[i,c] = msg_df.loc[i,c]

    # for ri in res:
    #     if ri in msg_df.columns:

    #         for i in msg_df.index:
    #             validation_dataset.loc[i,ri] = msg_df.loc[i,ri]

    validation_dataset = pd.DataFrame.from_dict(msg_df)
    validation_dataset.fillna(0, inplace=True)
    # validation_dataset = validation_dataset.loc[(validation_dataset!=0).any(axis=1)]

    right_index = []
    for pool in pools:
        if len(pool) == 1:
            not_c = [ci for ci in act if ci != pool[0]]
            zero_ind = validation_dataset[not_c][
                (validation_dataset[not_c] == 0).all(axis=1)
            ].index
            for i in validation_dataset.index:
                if (validation_dataset.loc[i, pool[0]] != 0) and (i in zero_ind):
                    right_index.append(i)
        else:
            not_c = [ci for ci in act if ci not in pool]
            zero_ind = validation_dataset[not_c][
                (validation_dataset[not_c] == 0).all(axis=1)
            ].index
            for i in validation_dataset.index:
                flag = True
                for p in pool:
                    if validation_dataset.loc[i, p] == 0:
                        flag = False
                if flag and (i in zero_ind):
                    right_index.append(i)
    validation_dataset = validation_dataset[act + res]
    new_val_dataset = pd.DataFrame(validation_dataset.loc[right_index, :])

    new_ind = []
    for i in new_val_dataset.index:
        new_ind.append(datetime.datetime.strptime(i, "%d.%m.%Y"))
    new_val_dataset.index = new_ind
    new_val_dataset = new_val_dataset.sort_index()
    new_ind = []
    for i in new_val_dataset.index:
        try:
            new_ind.append(datetime.datetime.strftime(i, "%d.%m.%Y"))
        except BaseException:
            print("bug")
    new_val_dataset.index = new_ind

    model_columns = []
    for c in new_val_dataset.columns:
        if c in act:
            model_columns.append(c + "_act_fact")
        if c in res:
            model_columns.append(c + "_res_fact")
    new_val_dataset.columns = model_columns

    return new_val_dataset



from spprval.dataset.BaseDataset import BaseDataset
import datetime
from datetime import timedelta
import pandas as pd
from spprval.dataset.ObjectNameProcessor import ObjectNameProcessor


class PlanDataset(BaseDataset):
    def __init__(self, ksg_data):
        self.ksg_data = ksg_data

    def collect(self):
        model_dataset = pd.DataFrame()
        for w in self.ksg_data["activities"]:
            name = w["activity_name"] + "_act_fact"
            start = datetime.datetime.strptime(w["start_date"].split()[0], "%Y-%m-%d")
            end = datetime.datetime.strptime(w["end_date"].split()[0], "%Y-%m-%d")
            vol = float(w["volume"])
            res_data = dict()
            for r in w["labor_resources"]:
                res_data[r["labor_name"]] = r["volume"]
            days = (end - start).days + 1
            vol_per = vol / days
            delta = timedelta(days=1)
            while start <= end:
                model_dataset.loc[start, name] = vol_per
                for k in res_data.keys():
                    model_dataset.loc[start, k + "_res_fact"] = res_data[k]
                start += delta
        model_dataset.fillna(0, inplace=True)
        model_dataset.index = model_dataset.index.strftime("%d.%m.%Y")
        return model_dataset

    def get_act_names(self):
        act = []
        for w in self.ksg_data["activities"]:
            act.append(w["activity_name"])
        return act

    def get_res_names(self):
        res = []
        for w in self.ksg_data["activities"]:
            for r in w["labor_resources"]:
                if r["labor_name"] not in res:
                    res.append(r["labor_name"])
        return res

    def get_pools(self):
        model_dataset = pd.DataFrame()
        for w in self.ksg_data["activities"]:
            name = w["activity_name"]
            start = datetime.datetime.strptime(w["start_date"].split()[0], "%Y-%m-%d")
            end = datetime.datetime.strptime(w["end_date"].split()[0], "%Y-%m-%d")
            vol = float(w["volume"])
            days = (end - start).days + 1
            vol_per = vol / days
            delta = timedelta(days=1)
            while start <= end:
                model_dataset.loc[start, name] = vol_per
                start += delta
        model_dataset.fillna(0, inplace=True)
        work_pools = []
        for i in model_dataset.index:
            pool = []
            for c in model_dataset.columns:
                if model_dataset.loc[i, c] != 0:
                    pool.append(c)
            if pool not in work_pools:
                work_pools.append(pool)

        # act_names = self.get_act_names()
        # object_name_processor = ObjectNameProcessor()
        # act_pr_dict = object_name_processor.create_processed_dict(
        #     names=act_names, name_type="act"
        # )

        # new_pools = []
        # for p in work_pools:
        #     pool = []
        #     for pi in p:
        #         if pi in act_pr_dict:
        #             if act_pr_dict[pi][0] not in pool:
        #                 pool.append(act_pr_dict[pi][0])
        #         else:
        #             if pi not in pool:
        #                 pool.append(pi)
        #     if pool not in new_pools:
        #         new_pools.append(pool)
        return work_pools



import warnings
from abc import ABC, abstractmethod

import pandas as pd

warnings.simplefilter(action="ignore", category=FutureWarning)
warnings.simplefilter(action="ignore", category=RuntimeWarning)


class BaseValidator(ABC):
    def __init__(
        self,
        percent_delta: float = 0.5,
        lower_quantile: float = 0.01,
        upper_quantile: float = 0.99,
    ):
        self.percent_delta = percent_delta
        self.lower_quantile = lower_quantile
        self.upper_quantile = upper_quantile

    @abstractmethod
    def validate(
        self, model_dataset: pd.DataFrame, df_wind_val: pd.DataFrame, act: list
    ):
        pass



from spprval.database.DataSource import DataSource
from idbadapter.schedule_loader import Schedules, PROCESSED, GRANULARY


class DBWrapper(DataSource):
    def __init__(self, url, level=GRANULARY):
        self.adapter = Schedules(url)
        self.level = level

    def get_data(self, pools, res_names):
        validation_dataset_list = []
        for df in self.adapter.get_works_by_pulls(
            work_pulls=pools, resource_list=res_names, key=self.level, res_key=GRANULARY
        ):
            if df is not None:
                df.fillna(0, inplace=True)
            validation_dataset_list.append(df)
        return validation_dataset_list

    def get_act_names(self):
        df = self.adapter.get_all_works_name()
        return df

    def get_res_names(self):
        df = self.adapter.get_all_resources_name()
        return df
